//////////////////////
//  COST FUNCTIONS  //
//////////////////////

//funcprot(0);

//exec('optimizations/simulated_annealing/optim_sa.sci',0);
exec('optimizations/optStrategies_functions.sci',0);

// calculate the economic cost
function K = cost(x)
    nodes = x(2).nodes;
    channels = x(2).abstractChannels;
    K = 0;
    for i=1:length(nodes)
        K = K + nodes(i).k;
    end
    for i=1:length(channels)
        K = K + channels(i).k;
    end    
endfunction

// calculate the power consumption
function P = powerConsumption(x)
    nodes = x(2).nodes;
    channels = x(2).abstractChannels;
    P = 0;
    for i=1:length(nodes)
        node = nodes(i);
        P = P + node.p;
        for j=1:length(node.t)
            task = node.t(j);
            for k=1:length(task.c)
                P = P + task.c(k)*node.g(k);
            end
        end
    end
    for i=1:length(channels)
        P = P + channels(i).p;
    end    
endfunction

// calculate the delay
function D = delay(x)
    nodes = x(2).nodes;
    channels = x(2).abstractChannels;
    zones= x(2).zones;
    contiguities = x(2).contiguities;
    D=0;
    for i=1:length(channels)
        for j=1:length(channels(i).f)
            D = D + channels(i).c.delay;
            contiguity = getContFromDataFlow(channels(i).f(j),nodes,zones,contiguities);
            if (contiguity <> -1) then
                D = D + contiguity.r.added_delay;
            end
        end
    end
endfunction

// calculate the error rate
function E = errorRate(x)
    nodes = x(2).nodes;
    channels = x(2).abstractChannels;
    zones= x(2).zones;
    contiguities = x(2).contiguities;
    E=0;
    nDataFlows = 0;
    for i=1:length(channels)
        for j=1:length(channels(i).f)
            nDataFlows = nDataFlows+1;
            E = E + channels(i).c.error_rate;
            contiguity = getContFromDataFlow(channels(i).f(j),nodes,zones,contiguities);
            if (contiguity <> -1) then
                E = E + contiguity.r.added_error_rate;
            end
        end
    end
    E = E/nDataFlows;
endfunction

///////////////////////////////
//  OPTIMIZATION OBJECTIVES  //
///////////////////////////////

// ----------------------------
//  ECONOMIC COST MINIMIZATION
// ----------------------------
// Minimize the global economic cost of the network
// by reducing the number of nodes and channels

function [manipulations_minCost,results_minCost,intResults_minCost] = minCost(network,Log)
    
    // NODES REMOVAL
    
    x0 = list(network,network,0,manipulations,manipulations);
    
    Proba_start = 0.7;
    It_Pre = 100;
    It_extern = 5;
    It_intern = 1000;
    
    saparams = init_param();
    saparams = add_param(saparams,"neigh_func", nodesMerging);  
    T0 = compute_initial_temp(x0, cost, Proba_start, It_Pre, saparams);
    
    if (Log) then
        mprintf("\n-----------------------------------------------------------");
        mprintf("\n- Economic cost minimization - PHASE 1 (Nodes Removal)    -");
        mprintf("\n-----------------------------------------------------------\n\n");
    end
    
    //[x_opt, f_opt, f_history_1, x_history_1] = optim_sa(x0, cost, It_extern, It_intern, T0, Log, saparams);
    [x_opt, f_opt] = optim_sa(x0, cost, It_extern, It_intern, T0, Log, saparams);
    
    network_opt = x_opt(2);
    manipulations_curr = x_opt(5);
        
    // CHANNELS REMOVAL
    
    x0 = list(network_opt,network_opt,0,manipulations_curr,manipulations_curr);
    
    saparams = set_param(saparams,"neigh_func", channelsMerging);    
    T0 = compute_initial_temp(x0, cost, Proba_start, It_Pre, saparams);

    if (Log) then
        mprintf("\n-----------------------------------------------------------");
        mprintf("\n- Economic cost minimization - PHASE 2 (Channels Removal) -");
        mprintf("\n-----------------------------------------------------------\n\n");
    end
    
    //[x_opt, f_opt, f_history_2, x_history_2] = optim_sa(x0, cost, It_extern, It_intern, T0, Log, saparams);
    [x_opt, f_opt] = optim_sa(x0, cost, It_extern, It_intern, T0, Log, saparams);
    
    //f_history = lstcat(f_history_1,f_history_2);
    //x_history = lstcat(x_history_1,x_history_2);
    
    network_opt = x_opt(2);
    manipulations = x_opt(5);

    manipulations_minCost = manipulations;
    results_minCost = struct('new_network',network_opt,'f_opt',f_opt);
    //intResults_minCost = struct('f',f_history,'x',x_history);
endfunction

// --------------------------------
//  POWER CONSUMPTION MINIMIZATION
// --------------------------------
// Minimize the global power consumption of the
// network by reallocating tasks in nodes

function [manipulations_minPower,results_minPower,intResults_minPower] = minPower(network,techLib,Log)
    
    // TASKS REALLOCATION
    
    x0 = list(network,network,0,manipulations,manipulations,techLib);
    
    Proba_start = 0.7;
    It_Pre = 100;
    It_extern = 5;
    It_intern = 1000;
    
    saparams = init_param();
    saparams = add_param(saparams,"neigh_func", tasksReallocation);    
    T0 = compute_initial_temp(x0, powerConsumption, Proba_start, It_Pre, saparams);

    if (Log) then
        mprintf("\n-----------------------------------------------------------------");
        mprintf("\n- Power consumption minimization - PHASE 1 (Tasks Reallocation) -");
        mprintf("\n-----------------------------------------------------------------\n\n");
    end
    
    //[x_opt, f_opt, f_history, x_history] = optim_sa(x0, powerConsumption, It_extern, It_intern, T0, Log, saparams);
    [x_opt, f_opt] = optim_sa(x0, powerConsumption, It_extern, It_intern, T0, Log, saparams);
    
    network_opt = x_opt(2);
    manipulations = x_opt(5);

    manipulations_minPower = manipulations;
    results_minPower = struct('new_network',network_opt,'f_opt',f_opt);
    //intResults_minPower = struct('f',f_history,'x',x_history);
endfunction

// --------------------------------
// TRANSMISSION DELAY MINIMIZATION 
// --------------------------------
// Minimize the global transmission delay of the
// network by reducing the number of tasks
// communicating with each other, reallocating tasks
// in nodes, and reallocating data flows in channels

function [manipulations_minDelay,results_minDelay, intResults_minDelay] = minDelay(network,techLib,Log)
    
    // DATA FLOWS REALLOCATION
    
    x0 = list(network,network,0,manipulations,manipulations,techLib);
    
    Proba_start = 0.7;
    It_Pre = 100;
    It_extern = 5;
    It_intern = 1000;
    
    saparams = init_param();
    saparams = add_param(saparams,"neigh_func", dataFlowsReallocation);    
    T0 = compute_initial_temp(x0, delay, Proba_start, It_Pre, saparams);

    if (Log) then
        mprintf("\n----------------------------------------------------------");
        mprintf("\n- Delay minimization - PHASE 1 (Data Flows Reallocation) -");
        mprintf("\n----------------------------------------------------------\n\n");
    end
    
    //[x_opt, f_opt, f_history, x_history] = optim_sa(x0, delay, It_extern, It_intern, T0, Log, saparams);
    [x_opt, f_opt] = optim_sa(x0, delay, It_extern, It_intern, T0, Log, saparams);
    
    network_opt = x_opt(2);
    manipulations = x_opt(5);

    manipulations_minDelay = manipulations;
    results_minDelay = struct('new_network',network_opt,'f_opt',f_opt);
    //intResults_minDelay = struct('f',f_history,'x',x_history);
endfunction

// -------------------------
//  ERROR RATE MINIMIZATION 
// -------------------------
// Minimize the global error rate of the network by
// reducing the number of tasks communicating with
// each other, reallocating tasks in nodes, and
// reallocating data flows in channels

function [manipulations_minError,results_minErrorRate,intResults_minErrorRate] = minErrorRate(network,techLib,Log)

    // DATA FLOWS REALLOCATION
    
    x0 = list(network,network,0,manipulations,manipulations,techLib);
    
    Proba_start = 0.7;
    It_Pre = 100;
    It_extern = 5;
    It_intern = 1000;
    
    saparams = init_param();
    saparams = add_param(saparams,"neigh_func", dataFlowsReallocation);    
    T0 = compute_initial_temp(x0, errorRate, Proba_start, It_Pre, saparams);

    if (Log) then
        mprintf("\n---------------------------------------------------------------");
        mprintf("\n- Error rate minimization - PHASE 1 (Data Flows Reallocation) -");
        mprintf("\n---------------------------------------------------------------\n\n");
    end
    
    //[x_opt, f_opt, f_history, x_history] = optim_sa(x0, errorRate, It_extern, It_intern, T0, Log, saparams);
    [x_opt, f_opt] = optim_sa(x0, errorRate, It_extern, It_intern, T0, Log, saparams);
    
    network_opt = x_opt(2);
    manipulations = x_opt(5);

    manipulations_minError = manipulations;
    results_minErrorRate = struct('new_network',network_opt,'f_opt',f_opt);
    //intResults_minErrorRate = struct('f',f_history,'x',x_history);
endfunction
